package frc.robot.subsystems.sysid;

import static edu.wpi.first.units.Units.Volts;
import static edu.wpi.first.units.Units.Radians;
import static edu.wpi.first.units.Units.RadiansPerSecond;

import edu.wpi.first.units.measure.MutAngle;
import edu.wpi.first.units.measure.MutAngularVelocity;
import edu.wpi.first.units.measure.MutVoltage;
import edu.wpi.first.wpilibj.RobotController;
import edu.wpi.first.wpilibj2.command.Command;
import edu.wpi.first.wpilibj2.command.sysid.SysIdRoutine;
import frc.robot.constants.ArmConstants;
import frc.robot.subsystems.Arm;

public class ArmSysID extends Arm {
    private MutVoltage appliedVoltage;

    private MutAngle armPosition;

    private MutAngularVelocity armVelocity;

    private SysIdRoutine routine;

    public ArmSysID() {
        super();

        appliedVoltage = Volts.mutable(0);

        armPosition = Radians.mutable(0);

        armVelocity = RadiansPerSecond.mutable(0);

        routine = new SysIdRoutine(
            ArmConstants.kSysIDConfig,
            new SysIdRoutine.Mechanism(
                armMotor::setVoltage, 
                (log) -> {
                    log.motor("armMotor")
                        .voltage(appliedVoltage.mut_replace(
                            armMotor.get() * RobotController.getBatteryVoltage(), Volts
                        ))
                        .angularPosition(armPosition.mut_replace(
                            getEncoderPosition(), Radians
                        ))
                        .angularVelocity(armVelocity.mut_replace(
                            getEncoderVelocity(), RadiansPerSecond
                        ));
                }, 
                this
            )
        );
    }

    public Command sysIdQuasistatic(SysIdRoutine.Direction direction) {
        return routine.quasistatic(direction);
    }

    public Command sysIdDynamic(SysIdRoutine.Direction direction) {
        return routine.dynamic(direction);
    }
}
